@@ -3,35 +3,45 @@
 
 """
 mapillary.utils.format
-~~~~~~~~~~~~~~~~~~~~~~
+======================
 
 This module deals with converting data to and from different file formats.
 
-:copyright: (c) 2021 Facebook
-:license: MIT LICENSE
+- Copyright: (c) 2021 Facebook
+- License: MIT LICENSE
 """
 
 # Package imports
-import mapbox_vector_tile
-import collections
-import typing
 import base64
+import collections
 import json
+import typing
+import mapbox_vector_tile
 
 # Local imports
-
-# Models
+# # Models
 from mapillary.models.geojson import GeoJSON
 
 
 def feature_to_geojson(json_data: dict) -> dict:
+    """
+    Converts feature into a GeoJSON, returns output
 
-    """From
-    {'geometry': {'type': 'Point', 'coordinates': [30.003755665554, 30.985948744314]}, 'id':
-    '506566177256016'}
-    To get,
-    {'type': 'FeatureCollection', 'features': [{'type': 'Feature', 'geometry': {'type': 'Point',
-    'coordinates': [30.98594605922699, 30.003757307208872]}, 'properties': {}}]}
+    From::
+
+        >>> {'geometry': {'type': 'Point', 'coordinates': [30.003755665554, 30.985948744314]},
+        ... 'id':'506566177256016'}
+
+    To::
+
+        >>> {'type': 'FeatureCollection', 'features': [{'type': 'Feature', 'geometry': {'type':
+        ... 'Point','coordinates': [30.98594605922699, 30.003757307208872]}, 'properties': {}}]}
+
+    :param json_data: The feature as a JSON
+    :type json_data: dict
+
+    :return: The formatted GeoJSON
+    :rtype: dict
     """
 
     # The geometry property will always be present
@@ -39,7 +49,6 @@ def feature_to_geojson(json_data: dict) -> dict:
 
     feature = {"type": "Feature", "geometry": {}, "properties": {}}
     # Make sure that all keys exist and retrieve their values if specified
-    feature["geometry"] = json_data["geometry"]
 
     for key in keys:
         feature["properties"][key] = json_data[key]
@@ -53,16 +62,33 @@ def join_geojson_with_keys(
     geojson_dest: dict,
     geojson_dest_key: str,
 ) -> dict:
+    """
+    Combines two GeoJSONS based on the similarity of their specified keys, similar to
+    the SQL join functionality
 
-    """Combines features from two geojsons on the basis of same given key ids, similar to
-    an SQL join functionality
+    :param geojson_src: The starting GeoJSO source
+    :type geojson_src: dict
+
+    :param geojson_src_key: The key in properties specified for the GeoJSON source
+    :type geojson_src_key: str
+
+    :param geojson_dest: The GeoJSON to merge into
+    :type geojson_dest: dict
+
+    :param geojson_dest_key: The key in properties specified for the GeoJSON to merge into
+    :type geojson_dest_key: dict
+
+    :return: The merged GeoJSON
+    :rtype: dict
 
     Usage::
+
         >>> join_geojson_with_keys(
-            geojson_src=geojson_src,
-            geojson_src_key='id',
-            geojson_dest=geojson_dest,
-            geojson_dest_key='id')
+        ...     geojson_src=geojson_src,
+        ...     geojson_src_key='id',
+        ...     geojson_dest=geojson_dest,
+        ...     geojson_dest_key='id'
+        ... )
     """
 
     # Go through the feature set in the src geojson
@@ -95,10 +121,9 @@ def join_geojson_with_keys(
                 # Going through the old properties in the features of src_geojson
                 for new_property in new_properties:
 
-                    # Going through the new propeties in the features of dest_geojson
+                    # Going through the new properties in the features of dest_geojson
                     if new_property not in old_properties:
-
-                        # Put the new_featu
+                        # Put the new_feature
                         from_features["properties"][new_property] = old_properties[
                             "properties"
                         ][new_property]
@@ -106,54 +131,74 @@ def join_geojson_with_keys(
     return geojson_src
 
 
-def geojson_to_features_list(json_data):
-    """Converts a decoded output GeoJSON to a list of feature objects
-    example:
-    From,
-    {'type': 'FeatureCollection', 'features': [{'type': 'Feature', 'geometry': {'type': 'Point',
-    'coordinates': [30.98594605922699, 30.003757307208872]}, 'properties': {}}]}
-
-    To,
-    [{'type': 'Feature', 'geometry': {'type': 'Point',
-    'coordinates': [30.98594605922699, 30.003757307208872]}, 'properties': {}}]
+def geojson_to_features_list(json_data: dict) -> list:
+    """
+    Converts a decoded output GeoJSON to a list of feature objects
 
     The purpose of this formatting utility is to obtain a list of individual features for
     decoded tiles that can be later extended to the output GeoJSON
+
+    From::
+
+        >>> {'type': 'FeatureCollection', 'features': [{'type': 'Feature', 'geometry':
+        ... {'type': 'Point','coordinates': [30.98594605922699, 30.003757307208872]},
+        ... 'properties': {}}]}
+
+    To::
+
+        >>> [{'type': 'Feature', 'geometry': {'type': 'Point',
+        ... 'coordinates': [30.98594605922699, 30.003757307208872]}, 'properties': {}}]
+
+    :param json_data: The given json data
+    :type json_data: dict
+
+    :return: The feature list
+    :rtype: list
     """
 
     return json_data["features"]
 
 
 def merged_features_list_to_geojson(features_list: list) -> str:
-    """Converts a processed features list (i.e. a features list with all the needed features merged
+    """
+    Converts a processed features list (i.e. a features list with all the needed features merged
     from multiple tiles) into a fully-featured GeoJSON
 
+    From::
+
+        >>> [{'type': 'Feature', 'geometry': {'type': 'Point',
+        ... 'coordinates': [30.98594605922699, 30.003757307208872]}, 'properties': {}}, ...]
+
+    To::
+
+        >>> {'type': 'FeatureCollection', 'features': [{'type': 'Feature', 'geometry':
+        ... {'type': 'Point','coordinates': [30.98594605922699, 30.003757307208872]},
+        ... 'properties': {}}, ...]}
+
     :param features_list: a list of processed features merged from different tiles within a bbox
     :type features_list: list
-    example:
-    From,
-    [{'type': 'Feature', 'geometry': {'type': 'Point',
-    'coordinates': [30.98594605922699, 30.003757307208872]}, 'properties': {}}, ...]
-
-    To,
-    {'type': 'FeatureCollection', 'features': [{'type': 'Feature', 'geometry': {'type': 'Point',
-    'coordinates': [30.98594605922699, 30.003757307208872]}, 'properties': {}}, ...]}
 
     :return: GeoJSON string formatted with all the extra commas removed.
     :rtype: str
     """
+
     return json.dumps({"type": "FeatureCollection", "features": features_list})
 
 
 def detection_features_to_geojson(feature_list: list) -> dict:
-    """Converts a preprocessed list (i.e, features from the detections of either images or
+    """
+    Converts a preprocessed list (i.e, features from the detections of either images or
     map_features from multiple segments) into a fully featured GeoJSON
 
-    :param features_list: A list of processed features merged from different segments within a
-    detection
-    :type features_list: list
+    :param feature_list: A list of processed features merged from different segments within a
+        detection
+    :type feature_list: list
+
+    :return: GeoJSON formatted as expected in a detection format
+    :rtype: dict
 
     Example::
+
         >>> # From
         >>> [{'created_at': '2021-05-20T17:49:01+0000', 'geometry':
         ... 'GjUKBm1weS1vchIVEgIAABgDIg0JhiekKBoqAABKKQAPGgR0eXBlIgkKB3BvbHlnb24ogCB4AQ==',
@@ -165,11 +210,8 @@ def detection_features_to_geojson(feature_list: list) -> dict:
         ... {'type': 'Point', 'coordinates': [-97.743279722222, 30.270651388889]}, 'properties': {
         ... 'image_id': '1933525276802129', 'created_at': '2021-05-20T17:49:01+0000',
         ... 'pixel_geometry':
-        ... 'GjUKBm1weS1vchIVEgIAABgDIg0JhiekKBoqAABKKQAPGgR0eXBlIgkKB3BvbHlnb24ogCB4AQ=='`,
+        ... 'GjUKBm1weS1vchIVEgIAABgDIg0JhiekKBoqAABKKQAPGgR0eXBlIgkKB3BvbHlnb24ogCB4AQ==',
         ... 'value': 'regulatory--no-parking--g2', 'id': '1942105415944115' } }, ... ]}
-
-    :return: GeoJSON formatted as expected in a detection format
-    :rtype: dict
     """
 
     resulting_geojson = {
@@ -219,39 +261,44 @@ def detection_features_to_geojson(feature_list: list) -> dict:
     for _feature in resulting_geojson["features"]:
 
         # Going through each property in the feature
-        for _property in _feature["properties"]:
+        for key, value in _feature["properties"].items():
 
-            # If the _property has defauled to None
-            if _property is None:
+            # If the _property has defaulted to None
+            if value is None:
 
                 # Delete the _property from the feature
-                del _feature["properties"][_property]
+                del _feature["properties"][key]
 
     # Finally return the output
     return resulting_geojson
 
 
-def flatten_geojson(geojson: dict) -> dict:
-    """Flattens a GeoJSON dictionary to a dictionary with only the relevant keys.
-    This is useful for writing to a CSV file. The output is a dictionary with the following
-    structure:
-    {
-        "geometry": {
-            "type": "Point",
-            "coordinates": [x, y]
-        },
-        "first_seen_at": "UNIX_TIMESTAMP",
-        "last_seen_at": "UNIX_TIMESTAMP",
-        "value": "regulatory--no-parking--g2",
-        "id": "FEATURE_ID",
-        "image_id": "IMAGE_ID"
-    }
+def flatten_geojson(geojson: dict) -> list:
+    """
+    Flattens a GeoJSON dictionary to a dictionary with only the relevant keys.
+    This is useful for writing to a CSV file.
+
+    Output Structure::
+
+        >>> {
+        ...     "geometry": {
+        ...         "type": "Point",
+        ...         "coordinates": [71.45343, 12.523432]
+        ...     },
+        ...     "first_seen_at": "UNIX_TIMESTAMP",
+        ...     "last_seen_at": "UNIX_TIMESTAMP",
+        ...     "value": "regulatory--no-parking--g2",
+        ...     "id": "FEATURE_ID",
+        ...     "image_id": "IMAGE_ID"
+        ... }
+
     :param geojson: The GeoJSON to flatten
     :type geojson: dict
+
     :return: A flattened GeoJSON
     :rtype: dict
 
-    Note:
+    Note,
         1. The `geometry` key is always present in the output
         2. The properties are flattened to the following keys:
             - "first_seen_at"   (optional)
@@ -263,8 +310,10 @@ def flatten_geojson(geojson: dict) -> dict:
         3. If the 'geometry` type is `Point`, two more properties will be added:
             - "longitude"
             - "latitude"
-        # TODO: Further testing needed with different geometries, e.g., Polygon, etc.
+
+    *TODO*: Further testing needed with different geometries, e.g., Polygon, etc.
     """
+
     for feature in geojson["features"]:
         # Check if the geometry is a Point
         if feature["geometry"]["type"] == "Point":
@@ -279,77 +328,79 @@ def flatten_geojson(geojson: dict) -> dict:
     ]
 
 
-def geojson_to_polgyon(geojson: dict) -> dict:
-    """Converts a GeoJSON into a collection of only geometry coordinates for the purpose of
-    checking whether a given coordinate point exists within a shapely polygon - see requirement #9
-    for more context
+def geojson_to_polygon(geojson: dict) -> GeoJSON:
+    """
+    Converts a GeoJSON into a collection of only geometry coordinates for the purpose of
+    checking whether a given coordinate point exists within a shapely polygon
 
-    Converts from,
-    '{
-        "type": "FeatureCollection",
-        "features": [
-            {
-                "geometry": {
-                    "coordinates": [
-                        -80.13069927692413,
-                        25.78523699486192
-                    ],
-                    "type": "Point"
-                },
-                "properties": {
-                    "first_seen_at": 1422984049000,
-                    "id": 481978503020355,
-                    "last_seen_at": 1422984049000,
-                    "value": "object--street-light"
-                },
-                "type": "Feature"
-            },
-            {
-                "geometry": {
-                    "coordinates": [
-                        -80.13210475444794,
-                        25.78362849816017
-                    ],
-                    "type": "Point"
-                },
-                "properties": {
-                    "first_seen_at": 1423228306666,
-                    "id": 252538103315239,
-                    "last_seen_at": 1423228306666,
-                    "value": "object--street-light"
-                },
-                "type": "Feature"
-            },
-            ...
-        ]
-    }'
+    From::
 
-    To,
-    '{
-        "type": "FeatureCollection",
-        "features": [
-            {
-            "type": "Feature",
-            "properties": {},
-            "geometry": {
-                "type": "Polygon",
-                "coordinates": [
-                [
-                    [
-                    7.2564697265625,
-                    43.69716905314008
-                    ],
-                    [
-                    7.27020263671875,
-                    43.69419030566581
-                    ],
-                    ...
-                ]
-                ]
-            }
-            }
-        ]
-    }'
+        >>> {
+        ...     "type": "FeatureCollection",
+        ...     "features": [
+        ...         {
+        ...             "geometry": {
+        ...                 "coordinates": [
+        ...                     -80.13069927692413,
+        ...                     25.78523699486192
+        ...                 ],
+        ...                 "type": "Point"
+        ...             },
+        ...             "properties": {
+        ...                 "first_seen_at": 1422984049000,
+        ...                 "id": 481978503020355,
+        ...                 "last_seen_at": 1422984049000,
+        ...                 "value": "object--street-light"
+        ...             },
+        ...             "type": "Feature"
+        ...         },
+        ...         {
+        ...             "geometry": {
+        ...                 "coordinates": [
+        ...                     -80.13210475444794,
+        ...                     25.78362849816017
+        ...                 ],
+        ...                 "type": "Point"
+        ...             },
+        ...             "properties": {
+        ...                 "first_seen_at": 1423228306666,
+        ...                 "id": 252538103315239,
+        ...                 "last_seen_at": 1423228306666,
+        ...                 "value": "object--street-light"
+        ...             },
+        ...             "type": "Feature"
+        ...         },
+        ...         ...
+        ...     ]
+        ... }
+
+    To::
+
+        >>> {
+        ... "type": "FeatureCollection",
+        ... "features": [
+        ...         {
+        ...             "type": "Feature",
+        ...             "properties": {},
+        ...             "geometry": {
+        ...                 "type": "Polygon",
+        ...                 "coordinates": [
+        ...                     [
+        ...                         [
+        ...                             7.2564697265625,
+        ...                             43.69716905314008
+        ...                         ],
+        ...                         [
+        ...                             7.27020263671875,
+        ...                             43.69419030566581
+        ...                         ],
+        ...                         ...
+        ...                     ]
+        ...                 ]
+        ...             }
+        ...         }
+        ...     ]
+        ... }
 
     :param geojson: The input GeoJSON
     :type geojson: dict
@@ -381,18 +432,25 @@ def geojson_to_polgyon(geojson: dict) -> dict:
     )
 
 
-def flatten_dictionary(data: dict, parent_key: str = "", sep: str = "_") -> dict:
-    """Flattens dictionaries
+def flatten_dictionary(
+    data: typing.Union[dict, collections.MutableMapping],
+    parent_key: str = "",
+    sep: str = "_",
+) -> dict:
+    """
+    Flattens dictionaries
 
-    From,
-    result = {'mpy-or': {'extent': 4096, 'version': 2, 'features': [{'geometry': {'type':
-    'Polygon', 'coordinates': [[[2402, 2776], [2408, 2776]]]}, 'properties': {}, 'id': 1,
-    'type': 3}]}}
+    From::
 
-    To,
-    {'mpy-or_extent': 4096, 'mpy-or_version': 2, 'mpy-or_features': [{'geometry': {'type':
-    'Polygon', 'coordinates': [[[2402, 2776], [2408, 2776]]]}, 'properties': {}, 'id': 1,
-    'type': 3}]}
+        >>> {'mpy-or': {'extent': 4096, 'version': 2, 'features': [{'geometry': {'type':
+        ... 'Polygon', 'coordinates': [[[2402, 2776], [2408, 2776]]]}, 'properties': {}, 'id': 1,
+        ... 'type': 3}]}}
+
+    To::
+
+        >>> {'mpy-or_extent': 4096, 'mpy-or_version': 2, 'mpy-or_features': [{'geometry':
+        ... {'type':'Polygon', 'coordinates': [[[2402, 2776], [2408, 2776]]]}, 'properties':
+        ... {}, 'id': 1,'type': 3}]}
 
     :param data: The dictionary itself
     :type data: dict
@@ -403,7 +461,7 @@ def flatten_dictionary(data: dict, parent_key: str = "", sep: str = "_") -> dict
     :param sep: The separator
     :type sep: str
 
-    :return: A flattend dictionary
+    :return: A flattened dictionary
     :rtype: dict
     """
 
@@ -429,19 +487,22 @@ def flatten_dictionary(data: dict, parent_key: str = "", sep: str = "_") -> dict
             # Append to the list
             items.append((new_key, value))
 
-    # Cnverting to dictionary, and returning
+    # Converting to dictionary, and returning
     return dict(items)
 
 
 def normalize_list(coordinates: list, width: int = 4096, height: int = 4096) -> list:
-    """Normalizes a list of coordinates with the respective width and the height
+    """
+    Normalizes a list of coordinates with the respective width and the height
 
     From::
-    >>> coordinates = [[[2402, 2776], [2408, 2776]]]
+
+        >>> [[[2402, 2776], [2408, 2776]]]
 
     To::
-    >>> normalize_list(coordinates)
-    ... # [[[0.58642578125, 0.677734375], [0.587890625, 0.677734375]]]
+
+        >>> normalize_list(coordinates)
+        ... # [[[0.58642578125, 0.677734375], [0.587890625, 0.677734375]]]
 
     :param coordinates: The coordinate list to normalize
     :type coordinates: list
@@ -467,7 +528,6 @@ def normalize_list(coordinates: list, width: int = 4096, height: int = 4096) ->
 
         # If it is already normalized ...
         if 0 <= coordinate_pair[0] <= 1 and 0 <= coordinate_pair[1] <= 1:
-
             # ... then append as is
             new_coordinates.append(coordinate_pair)
 
@@ -484,7 +544,8 @@ def normalize_list(coordinates: list, width: int = 4096, height: int = 4096) ->
 def decode_pixel_geometry(
     base64_string: str, normalized: bool = True, width: int = 4096, height: int = 4096
 ) -> dict:
-    """Decodes the pixel geometry, and return the coordinates, which can be specified to be
+    """
+    Decodes the pixel geometry, and return the coordinates, which can be specified to be
     normalized
 
     :param base64_string: The pixel geometry encoded as a vector tile
@@ -509,18 +570,17 @@ def decode_pixel_geometry(
     # Getting the results from mapbox_vector_tile
     result = mapbox_vector_tile.decode(data)
 
-    # Flattening the resultant dicionary
+    # Flattening the resultant dictionary
     flattened = flatten_dictionary(result)
 
     # Init geometry var
     geometry = None
 
-    # For key in the flattend keys
+    # For key in the flattened keys
     for key in flattened.keys():
 
         # If the key contains the word "features"
         if "features" in key:
-
             # Get the geometry
             geometry = flattened[key][0]
 
@@ -543,7 +603,8 @@ def decode_pixel_geometry_in_geojson(
     width: int = 4096,
     height: int = 4096,
 ) -> GeoJSON:
-    """Decodes all the pixel_geometry
+    """
+    Decodes all the pixel_geometry
 
     :param geojson: The GeoJSON representation to be decoded
 
@@ -564,9 +625,8 @@ def decode_pixel_geometry_in_geojson(
     # Remove pass by reference for idempotent principle
     data = geojson.copy()
 
-    # Iterature through the features
+    # Iterate through the features
     for feature in data["features"]:
-
         # For the pixel_geometry feature, decode it
         feature["properties"]["pixel_geometry"] = decode_pixel_geometry(
             # Get the base64_string
